#include "DolibarrClient.h"
#include <WiFi.h>
#include <ArduinoJson.h>
#include <iostream>

DolibarrClient::DolibarrClient(struct DolibarrConfig dolibarr_config) : dolibarr(dolibarr_config) {
    #if defined(DEBUG)
        Serial.println(" --- Dolibarr configuration --- ");
        Serial.println((std::string("Base URL: ") + std::string(dolibarr_config.url)).c_str());
        Serial.println((std::string("Token: ") + std::string(dolibarr_config.api_key)).c_str());
    #endif
    this->initialize_http_client();
}

HTTPClient *DolibarrClient::build_url(const String& url) const {
    auto *client = new HTTPClient();
    String clientUrl = this->dolibarr.url + url;
    client->begin(clientUrl);
    client->addHeader("Content-Type", "application/json");
    client->addHeader("DOLAPIKEY", this->dolibarr.api_key);
    #if defined(DEBUG)
        Serial.println("URL Request: " + clientUrl);
    #endif
    return client;
}


int DolibarrClient::login() const {
    HTTPClient *client = this->build_url(API_LOGIN_URL);
    int httpResponseCode = client->GET();
    if (httpResponseCode > 0) {
        StaticJsonDocument<API_MAX_JSON_SIZE> doc;
        DeserializationError error = deserializeJson(doc, client->getString().c_str());
        if (error) {
            delete client;
            return -1;
        }
        delete client;
        return 0;
    }
    delete client;
    return -1;
}

String replace_id(const char *str, const char *id) {
    String url(str);
    url.replace("{id}", id);
    return url;
}

std::vector<models::Product> *DolibarrClient::list_products() const {
    HTTPClient *client = this->build_url(API_LIST_PRODUCT_URL);
    if (client->GET() == HTTP_CODE_OK) {

    }
    return nullptr;
}

models::Product *DolibarrClient::get_product_by_id(const char* id_product) const {
    HTTPClient *client = this->build_url(replace_id(API_GET_PRODUCT_URL, id_product).c_str());
    if (client->GET() == HTTP_CODE_OK) {
        StaticJsonDocument<API_MAX_JSON_SIZE> doc;
        DeserializationError error = deserializeJson(doc, client->getString().c_str());
        if (error) {
            Serial.println("ERROR: ");
            Serial.println(error.c_str());
            delete client;
            return nullptr;
        }
        auto *product = new models::Product();
        product->date_creation = doc["date_creation"];
        product->id = doc["id"];
        product->entity = doc["entity"];
        product->stock_reel = doc["stock_reel"];
        product->label = doc["label"];
        delete client;
        return product;
    }
    delete client;
    return nullptr;
}

std::vector<models::Warehouse> *DolibarrClient::list_warehouse() const {
    HTTPClient *client = this->build_url(API_LIST_WAREHOUSE_URL);
    if (client->GET() == HTTP_CODE_OK) {
        StaticJsonDocument<API_MAX_JSON_SIZE> doc;
        DeserializationError error = deserializeJson(doc, client->getString().c_str());
        if (error) {
            Serial.println("ERROR: ");
            Serial.println(error.c_str());
            delete client;
            return nullptr;
        }
        auto *warehouses = new std::vector<models::Warehouse>();
        for (auto obj : doc.as<JsonArray>()) {
            models::Warehouse warehouse = {};
            warehouse.id = obj["id"];
            warehouses->push_back(warehouse);
        }
        delete client;
        return warehouses;
    }
    delete client;
    return nullptr;
}

int DolibarrClient::create_movement(models::CreateProductStock &stock) const {
    HTTPClient *client = this->build_url("API_CREATE_STOCKS_MOVEMENTS_URL");
    StaticJsonDocument<API_MAX_JSON_SIZE> doc;
    std::string result;
    doc["product_id"] = stock.product_id;
    doc["warehouse_id"]  = stock.warehouse_id;
    doc["qty"] = stock.qty;
    serializeJson(doc, result);
    Serial.println(result.c_str());
    if (client->POST(result.c_str()) == HTTP_CODE_OK) {
        delete client;
        return 0;
    }
    return -1;
}

int DolibarrClient::initialize_http_client() {
    this->httpClient = new HTTPClient();
    if (this->login() == 0) {
        auto* product = this->get_product_by_id("1");
        if (product == nullptr) {
            Serial.println("Product is nullptr !");
            return -1;
        }
        Serial.println("Product label: ");
        Serial.println(product->label);
        auto* warehouses = this->list_warehouse();
        if (warehouses == nullptr) {
            delete product;
            Serial.println("Warehouse is nullptr !");
            return -1;
        }
        Serial.println("Warehouses: ");
        models::CreateProductStock product_stock = {product->id, warehouses->at(0).id, "1"};
        this->create_movement(product_stock);
        for (auto warehouse : *warehouses) {
            Serial.println(warehouse.id);
        }
        delete product;
        delete warehouses;
    } else {
        Serial.println("An Error has occurred while trying to login");
    }
    return 0;
}